import { A as Agent, M as MastraStorage, a as MastraMemory } from '../base-DqRXkxCw.js';
import { L as Logger, B as BaseLogMessage } from '../index-mKY1XrpK.js';
import { W as Workflow } from '../workflow-fGgxPZk4.js';
import { MastraVector } from '../vector/index.js';
import { O as OtelConfig, T as Telemetry } from '../base-D90KQ4XI.js';
import { MastraTTS } from '../tts/index.js';
import { MastraDeployer } from '../deployer/index.js';
import 'ai';
import 'sift';
import 'zod';
import 'json-schema';
import '../types-m9RryK9a.js';
import '../voice/index.js';
import 'stream';
import 'pino';
import '@opentelemetry/api';
import '../vector/filter/index.js';
import '@opentelemetry/sdk-trace-base';
import '../bundler/index.js';

interface Config<TAgents extends Record<string, Agent<any>> = Record<string, Agent<any>>, TWorkflows extends Record<string, Workflow> = Record<string, Workflow>, TVectors extends Record<string, MastraVector> = Record<string, MastraVector>, TTTS extends Record<string, MastraTTS> = Record<string, MastraTTS>, TLogger extends Logger = Logger> {
    agents?: TAgents;
    storage?: MastraStorage;
    vectors?: TVectors;
    logger?: TLogger | false;
    workflows?: TWorkflows;
    tts?: TTTS;
    telemetry?: OtelConfig;
    deployer?: MastraDeployer;
    memory?: MastraMemory;
}
declare class Mastra<TAgents extends Record<string, Agent<any>> = Record<string, Agent<any>>, TWorkflows extends Record<string, Workflow> = Record<string, Workflow>, TVectors extends Record<string, MastraVector> = Record<string, MastraVector>, TTTS extends Record<string, MastraTTS> = Record<string, MastraTTS>, TLogger extends Logger = Logger> {
    private vectors?;
    private agents;
    private logger;
    private workflows;
    private telemetry?;
    private tts?;
    private deployer?;
    storage?: MastraStorage;
    memory?: MastraMemory;
    constructor(config?: Config<TAgents, TWorkflows, TVectors, TTTS, TLogger>);
    getAgent<TAgentName extends keyof TAgents>(name: TAgentName): TAgents[TAgentName];
    getAgents(): TAgents;
    getVector<TVectorName extends keyof TVectors>(name: TVectorName): TVectors[TVectorName];
    getVectors(): TVectors | undefined;
    getDeployer(): MastraDeployer | undefined;
    getWorkflow<TWorkflowId extends keyof TWorkflows>(id: TWorkflowId, { serialized }?: {
        serialized?: boolean;
    }): TWorkflows[TWorkflowId];
    getWorkflows(props?: {
        serialized?: boolean;
    }): Record<string, Workflow>;
    setStorage(storage: MastraStorage): void;
    setLogger({ logger }: {
        logger: TLogger;
    }): void;
    setTelemetry(telemetry: OtelConfig): void;
    getLogger(): TLogger;
    getTelemetry(): Telemetry | undefined;
    getLogsByRunId({ runId, transportId }: {
        runId: string;
        transportId: string;
    }): Promise<BaseLogMessage[] | undefined>;
    getLogs(transportId: string): Promise<BaseLogMessage[]>;
}

export { type Config, Mastra };
