import { M as MastraBase } from '../base-hs9NDAZ2.cjs';
import { VectorFilter } from './filter/index.cjs';
import '@opentelemetry/api';
import '../index-mKY1XrpK.cjs';
import 'stream';
import 'pino';
import '@opentelemetry/sdk-trace-base';

interface QueryResult {
    id: string;
    score: number;
    metadata?: Record<string, any>;
    vector?: number[];
    /**
     * The document content, if available.
     * Note: Currently only supported by Chroma vector store.
     * For other vector stores, documents should be stored in metadata.
     */
    document?: string;
}
interface IndexStats {
    dimension: number;
    count: number;
    metric?: 'cosine' | 'euclidean' | 'dotproduct';
}
interface UpsertVectorParams {
    indexName: string;
    vectors: number[][];
    metadata?: Record<string, any>[];
    ids?: string[];
}
interface CreateIndexParams {
    indexName: string;
    dimension: number;
    metric?: 'cosine' | 'euclidean' | 'dotproduct';
}
interface QueryVectorParams {
    indexName: string;
    queryVector: number[];
    topK?: number;
    filter?: VectorFilter;
    includeVector?: boolean;
}
type ParamsToArgs<T> = [string | T, ...Array<T[Exclude<keyof T, 'indexName'>]>];

declare abstract class MastraVector extends MastraBase {
    constructor();
    private readonly baseKeys;
    protected normalizeArgs<T>(method: string, [first, ...rest]: ParamsToArgs<T>, extendedKeys?: string[]): T;
    abstract query(...args: ParamsToArgs<QueryVectorParams>): Promise<QueryResult[]>;
    abstract upsert(...args: ParamsToArgs<UpsertVectorParams>): Promise<string[]>;
    abstract createIndex(...args: ParamsToArgs<CreateIndexParams>): Promise<void>;
    abstract listIndexes(): Promise<string[]>;
    abstract describeIndex(indexName: string): Promise<IndexStats>;
    abstract deleteIndex(indexName: string): Promise<void>;
}

export { type CreateIndexParams, type IndexStats, MastraVector, type ParamsToArgs, type QueryResult, type QueryVectorParams, type UpsertVectorParams };
